﻿Option Explicit On

Public Class DlgKeyWord

    Private d_myXmlResult As System.Xml.XmlNode = Nothing


    Public Function ShowKeyEdit(ByVal avXmlNode As System.Xml.XmlNode, Optional ByRef apOwnerWindow As System.Windows.Forms.IWin32Window = Nothing) As System.Xml.XmlNode
        Try
            ' ========================================================
            '   XMLノードをインポートする
            ' ========================================================
            Dim myXmlDoc As System.Xml.XmlDocument = New System.Xml.XmlDocument
            Try
                d_myXmlResult = myXmlDoc.ImportNode(avXmlNode, True)
            Catch ex As Exception
                d_myXmlResult = Nothing
            End Try

            ' ========================================================
            '   ダイアログを表示してXMLを編集
            ' ========================================================
            Me.ShowDialog(apOwnerWindow)

            ' ========================================================
            '   XML編集失敗(キャンセルされたかも)
            ' ========================================================
            If d_myXmlResult Is Nothing Then Throw New ApplicationException
        Catch ex As Exception
            Return Nothing
        End Try

        Return d_myXmlResult
    End Function



    Private Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK_Button.Click
        ' ========================================================
        '   リストビューをXMLに変換
        ' ========================================================
        d_myXmlResult = GetXmlData()

        ' ========================================================
        '   ウィンドウを閉じる
        ' ========================================================
        Me.Close()
    End Sub


    Private Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        d_myXmlResult = Nothing
        Me.Close()
    End Sub



    Private Sub DlgKeyWord_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        KeyWordList.Clear()
    End Sub


    Private Sub DlgKeyWord_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ========================================================
        '   一度リストビューをクリアします
        ' ========================================================
        KeyWordList.Clear()

        ' ========================================================
        '   リストビューにカラムヘッダを追加します
        ' ========================================================
        With KeyWordList
            .View = View.Details
            KeyWordList.Columns.Add("ColumnKeyWord", "キーワード", 100)
            KeyWordList.Columns.Add("ColumnSearchType", "検索タイプ", 200)
            KeyWordList.Columns.Add("ColumnViewType", "表示タイプ", 170)
        End With

        ' ========================================================
        '   XMLをロードしてみる
        ' ========================================================
        LoadXmlData(d_myXmlResult)
    End Sub


    Private Sub btnAdd_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAdd.Click
        ' ========================================================
        '   ダイアログを表示して新しい項目を入力してもらいます
        ' ========================================================
        Dim myItem As dlgKeyProperty.KEYWORDTYPE = dlgKeyProperty.ShowAddDialog(Me)
        If System.String.IsNullOrEmpty(myItem.Keyword) Then
            Return
        End If

        ' ========================================================
        '   新しい項目をリストビューに追加します
        ' ========================================================
        AddListView(myItem)
    End Sub




    Private Sub btnProperty_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnProperty.Click
        If KeyWordList.SelectedItems Is Nothing Then Return
        If KeyWordList.SelectedItems.Count <= 0 Then Return

        Dim mySearchTypeIndex As Integer
        Dim myViewTypeIndex As Integer

        Try
            mySearchTypeIndex = KeyWordList.Columns("ColumnSearchType").DisplayIndex
            myViewTypeIndex = KeyWordList.Columns("ColumnViewType").DisplayIndex
        Catch ex As Exception
            Return
        End Try


        Dim myKeyWord As dlgKeyProperty.KEYWORDTYPE

        With KeyWordList.SelectedItems(0)
            myKeyWord.Keyword = .Text
            myKeyWord.Search = .SubItems(mySearchTypeIndex).Tag
            myKeyWord.KeyShowType = .SubItems(myViewTypeIndex).Text
        End With


        myKeyWord = dlgKeyProperty.ShowProperty(myKeyWord, Me)

        If Not System.String.IsNullOrEmpty(myKeyWord.Keyword) Then
            With KeyWordList.SelectedItems(0)
                .Text = myKeyWord.Keyword
                .SubItems(mySearchTypeIndex).Tag = myKeyWord.Search
                GetSearchTypeString(myKeyWord.Search, .SubItems(mySearchTypeIndex).Text)
                .SubItems(myViewTypeIndex).Text = myKeyWord.KeyShowType
            End With
        End If
    End Sub




    Private Sub btnDelete_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDelete.Click
        If KeyWordList.SelectedItems Is Nothing Then Return
        If KeyWordList.SelectedItems.Count <= 0 Then Return

        KeyWordList.SelectedItems(0).Remove()
    End Sub




    Private Function GetSearchTypeString(ByVal avSearchType As dlgKeyProperty.SEARCHTYPE, ByRef apGetString As String) As Boolean
        ' ========================================================
        '   検索方法を文字列に変換します
        ' ========================================================
        Dim mySearchString As String = vbNullString
        Select Case avSearchType
            Case dlgKeyProperty.SEARCHTYPE.Regular
                mySearchString = "正規表現検索"
            Case dlgKeyProperty.SEARCHTYPE.Word
                mySearchString = "単語検索(大文字/小文字区別)"
            Case dlgKeyProperty.SEARCHTYPE.WordInsensitive
                mySearchString = "単語検索(大文字/小文字同一視)"
            Case Else
        End Select

        If System.String.IsNullOrEmpty(mySearchString) Then
            Return False
        End If

        apGetString = mySearchString
        Return True
    End Function






    Private Function GetXmlData() As System.Xml.XmlNode
        Dim myXmlDoc As System.Xml.XmlDocument = New System.Xml.XmlDocument
        Dim myXmlRoot As System.Xml.XmlNode = Nothing

        Dim mySearchTypeIndex As Integer
        Dim myViewTypeIndex As Integer

        Try
            mySearchTypeIndex = KeyWordList.Columns("ColumnSearchType").DisplayIndex
            myViewTypeIndex = KeyWordList.Columns("ColumnViewType").DisplayIndex
        Catch ex As Exception
            Return Nothing
        End Try


        Dim myKeyWord As dlgKeyProperty.KEYWORDTYPE = Nothing



        ' ========================================================
        '   ルートノードを生成して選択
        ' ========================================================
        myXmlDoc.LoadXml("<?xml version=""1.0"" encoding=""utf-8"" ?>" & "<KEYWORDS/>")
        myXmlRoot = myXmlDoc.SelectSingleNode("./KEYWORDS")

        Try
            With KeyWordList
                For i As Integer = 0 To .Items.Count - 1
                    ' ========================================================
                    '   [ITEM]ノードを追加
                    ' ========================================================
                    Dim myXmlNode As System.Xml.XmlNode = myXmlRoot.AppendChild(myXmlDoc.CreateElement("ITEM"))

                    ' ========================================================
                    '   [KEY]ノードを生成して、キーワードを追加
                    ' ========================================================
                    Dim myXmlSubNode As System.Xml.XmlNode = myXmlNode.AppendChild(myXmlDoc.CreateElement("KEY"))
                    myXmlSubNode.InnerText = .Items(i).Text

                    ' ========================================================
                    '   [SEARCHTYPE]ノードを生成して、検索タイプを追加
                    ' ========================================================
                    myXmlSubNode = myXmlNode.AppendChild(myXmlDoc.CreateElement("SEARCHTYPE"))
                    myXmlSubNode.InnerText = System.Convert.ToInt32(.Items(i).SubItems(mySearchTypeIndex).Tag).ToString

                    ' ========================================================
                    '   [CSSTAGNAME]ノードを生成して、表示タイプを追加
                    ' ========================================================
                    myXmlSubNode = myXmlNode.AppendChild(myXmlDoc.CreateElement("CSSTAGNAME"))
                    myXmlSubNode.InnerText = .Items(i).SubItems(myViewTypeIndex).Text
                Next
            End With
        Catch ex As Exception
            Return Nothing
        End Try

        Return (myXmlRoot)
    End Function


    Private Function AddListView(ByVal avAddItem As dlgKeyProperty.KEYWORDTYPE) As Boolean

        ' ====== 重複するアイテム
        Dim myRemoveList As System.Windows.Forms.ListViewItem = Nothing
        Try

            With KeyWordList

                ' ========================================================
                '   重複する項目があればそれを覚えておきます。あとで削除します
                ' ========================================================
                For i As Integer = 0 To .Items.Count - 1
                    If .Items(i).Text = avAddItem.Keyword Then
                        myRemoveList = .Items(i)
                        Exit For
                    End If
                Next


                ' ========================================================
                '   キーワードをリストビューに追加します
                ' ========================================================
                Dim myList As System.Windows.Forms.ListViewItem = .Items.Add(avAddItem.Keyword)


                ' ========================================================
                '   検索方法を文字列に変換します
                ' ========================================================
                Dim mySearchString As String = vbNullString
                If Not GetSearchTypeString(avAddItem.Search, mySearchString) Then
                    Return (False)
                End If


                ' ========================================================
                '   検索方法表示用のサブアイテムを作成ます
                '   タグとして検索方法のENUM値を保存します
                ' ========================================================
                Dim myListSubItem As System.Windows.Forms.ListViewItem.ListViewSubItem = New System.Windows.Forms.ListViewItem.ListViewSubItem
                myListSubItem.Text = mySearchString
                myListSubItem.Tag = avAddItem.Search



                ' ========================================================
                '   生成したサブアイテムをリストに追加します
                ' ========================================================
                myList.SubItems.Add(myListSubItem)


                ' ========================================================
                '   キーワード表示種別をリストビューに追加します
                ' ========================================================
                myList.SubItems.Add(avAddItem.KeyShowType)


                ' ========================================================
                '   重複していた項目を削除します
                ' ========================================================
                If Not myRemoveList Is Nothing Then
                    .Items.Remove(myRemoveList)
                End If
            End With

        Catch ex As Exception
            Return False
        End Try

        Return True
    End Function



    Private Function LoadXmlData(ByVal avXmlNode As System.Xml.XmlNode) As Boolean

        ' ========================================================
        '   [ITEM]ノードを選択
        ' ========================================================
        Dim myXmlNodeList As System.Xml.XmlNodeList = Nothing
        Try
            myXmlNodeList = avXmlNode.SelectNodes("./ITEM")
        Catch ex As Exception
            Return (False)
        End Try

        ' ========================================================
        '   全ての[ITEM]ノードを抽出してリストビューに追加
        ' ========================================================
        For i As Integer = 0 To myXmlNodeList.Count - 1
            Try

                ' ========================================================
                '   XMLからデータを抽出します
                ' ========================================================
                Dim myKeyword As dlgKeyProperty.KEYWORDTYPE
                With myXmlNodeList(i)
                    ' ====== キーワードを取得
                    myKeyword.Keyword = .SelectSingleNode("./KEY").InnerText
                    ' ====== 表示種別を取得
                    myKeyword.KeyShowType = .SelectSingleNode("./CSSTAGNAME").InnerText
                    ' ====== 検索種別を取得
                    Dim myString As String = .SelectSingleNode("./SEARCHTYPE").InnerText
                    myKeyword.Search = System.Convert.ToInt32(myString)

                End With

                ' ========================================================
                '   抽出したデータをリストビューに追加します
                ' ========================================================
                If Not AddListView(myKeyword) Then Throw New ApplicationException
            Catch ex As Exception
            End Try
        Next

        Return (True)
    End Function

    Private Sub KeyWordList_MouseDoubleClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles KeyWordList.MouseDoubleClick

        ' ========================================================
        '   ダブルクリックされたら、プロパティボタンと同じ動作をしてみる
        ' ========================================================
        With KeyWordList
            If .SelectedItems.Count > 0 Then
                btnProperty_Click(sender, Nothing)
            End If
        End With
    End Sub

    Private Sub btnSaveAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSaveAs.Click

        Try
            ' ========================================================
            '   XMLドキュメントを作成
            ' ========================================================
            Dim myXmlDoc As System.Xml.XmlDocument = New System.Xml.XmlDocument
            myXmlDoc.LoadXml("<?xml version=""1.0"" encoding=""utf-8"" ?><ROOT/>")

            ' ========================================================
            '   キーワードノードをインポート
            ' ========================================================
            Dim myXmlRootNode As System.Xml.XmlNode = myXmlDoc.SelectSingleNode("./ROOT")
            myXmlRootNode.AppendChild(myXmlDoc.ImportNode(GetXmlData(), True))


            ' ========================================================
            '   保存ファイル選択ダイアログのインスタンスを生成
            ' ========================================================
            Dim mySaveDialog As System.Windows.Forms.SaveFileDialog = New System.Windows.Forms.SaveFileDialog
            mySaveDialog.Filter = "XMLファイル(*.xml)|*.xml|すべてのファイル(*.*)|*.*"
            mySaveDialog.Title = "保存ファイルを選択してください"
            mySaveDialog.RestoreDirectory = True


            ' ========================================================
            '   ダイアログを表示して保存ファイルを入力してもらう
            ' ========================================================
            If mySaveDialog.ShowDialog() <> System.Windows.Forms.DialogResult.OK Then Return


            ' ========================================================
            '   XMLファイルを保存
            ' ========================================================
            myXmlDoc.Save(mySaveDialog.FileName)

        Catch ex As Exception
        End Try
    End Sub

    Private Sub btnLoadAs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLoadAs.Click
        Try

            ' ========================================================
            '   ファイル選択ダイアログのインスタンスを生成
            ' ========================================================
            Dim myLoadDialog As System.Windows.Forms.OpenFileDialog = New System.Windows.Forms.OpenFileDialog
            myLoadDialog.Filter = "XMLファイル(*.xml)|*.xml|すべてのファイル(*.*)|*.*"
            myLoadDialog.Title = "キーワードファイルを選択してください"
            myLoadDialog.RestoreDirectory = True


            ' ========================================================
            '   ダイアログを表示してファイルを選択してもらう
            ' ========================================================
            If myLoadDialog.ShowDialog() <> System.Windows.Forms.DialogResult.OK Then Return

            ' ========================================================
            '   XMLドキュメントを作成して選択したファイルをロード
            ' ========================================================
            Dim myXmlDoc As System.Xml.XmlDocument = New System.Xml.XmlDocument
            myXmlDoc.Load(myLoadDialog.FileName)

            ' ========================================================
            '   ルートノードを選択して、XMLキーワードデータとしてロード
            ' ========================================================
            Dim myXmlRootNode As System.Xml.XmlNode = myXmlDoc.SelectSingleNode("./ROOT")
            If Not LoadXmlData(myXmlRootNode.SelectSingleNode("./KEYWORDS")) Then Throw New ApplicationException

        Catch ex As Exception
            MessageBox.Show("ロードに失敗しました", "警告", MessageBoxButtons.OK, MessageBoxIcon.Warning)
        End Try

    End Sub

    Private Sub btnListUp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnListUp.Click
        ' ========================================================
        '   何も選択されていないか、リストに２個以上項目がなければ終了
        ' ========================================================
        If KeyWordList.SelectedItems.Count <= 0 Then Return
        If KeyWordList.Items.Count <= 1 Then Return

        ' ====== リスト先頭が選択されていた場合は終了
        If KeyWordList.SelectedItems(0).Index <= 0 Then
            KeyWordList.Select()
            Return
        End If


        ' ========================================================
        '   選択されているリストをひとつ上に入れ替える
        ' ========================================================
        With KeyWordList

            ' ====== 選択されているアイテムを取得
            Dim mySelectList As System.Windows.Forms.ListViewItem = .SelectedItems(0)
            Dim mySelectIndex As Integer = mySelectList.Index

            '====== 自分を削除
            .Items.Remove(mySelectList)

            ' ====== 1個上に自分のコピーを挿入し、選択状態へ。
            .Items.Insert(mySelectIndex - 1, mySelectList).Selected = True
            .EnsureVisible(mySelectIndex - 1)
            .Select()
        End With
    End Sub

    Private Sub btnListDown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnListDown.Click
        ' ========================================================
        '   何も選択されていないか、リストに２個以上項目がなければ終了
        ' ========================================================
        If KeyWordList.SelectedItems.Count <= 0 Then Return
        If KeyWordList.Items.Count <= 1 Then Return

        ' ====== リスト先頭が選択されていた場合は終了
        If KeyWordList.SelectedItems(0).Index >= (KeyWordList.Items.Count - 1) Then
            KeyWordList.Select()
            Return
        End If

        ' ========================================================
        '   選択されているリストをひとつ上に入れ替える
        ' ========================================================
        With KeyWordList

            ' ====== 選択されているアイテムを取得
            Dim mySelectList As System.Windows.Forms.ListViewItem = .SelectedItems(0)
            Dim mySelectIndex As Integer = mySelectList.Index

            '====== 自分を削除
            .Items.Remove(mySelectList)

            ' ====== 1個下に自分のコピーを挿入し、選択状態へ。
            .Items.Insert(mySelectIndex + 1, mySelectList).Selected = True
            .EnsureVisible(mySelectIndex + 1)
            .Select()
        End With
    End Sub
End Class
