﻿Option Explicit On

Public Class dlgKeyProperty


    ' ========================================================================
    '   ◆キーワード検索種別
    ' ========================================================================
    ''' <summary>キーワード検索種別</summary>
    Public Enum SEARCHTYPE
        ''' <summary>単語検索(大文字/小文字区別する)</summary>
        Word = 0

        ''' <summary>単語検索(大文字/小文字区別しない)</summary>
        WordInsensitive

        ''' <summary>正規表現検索</summary>
        Regular
    End Enum


    ' ========================================================================
    '   ◆キーワードデータ構造体
    ' ========================================================================
    ''' <summary>設定データ</summary>
    Public Structure KEYWORDTYPE
        ''' <summary>キーワード</summary>
        Dim Keyword As String

        ''' <summary>検索タイプ</summary>
        Dim Search As SEARCHTYPE

        ''' <summary>キーワードの表示種別</summary>
        Dim KeyShowType As String
    End Structure



    ' ========================================================================
    '   ◆グローバル変数：既存データ
    ' ========================================================================
    Private Structure MYDEFAULT
        Dim Value As KEYWORDTYPE
        Dim IsEnable As Boolean
    End Structure
    Dim d_DefaultValue As MYDEFAULT

    ' ========================================================================
    '   ◆グローバル変数：キーワードデータ
    ' ========================================================================
    Dim d_myResult As KEYWORDTYPE = Nothing


    Public Function ShowAddDialog(Optional ByRef owner As System.Windows.Forms.IWin32Window = Nothing) As KEYWORDTYPE

        Dim myResult As System.Windows.Forms.DialogResult

        ' ========================================================
        '   デフォルト値を使用しない
        ' ========================================================
        d_DefaultValue.IsEnable = False

        ' ========================================================
        '   ダイアログを表示してキーワード情報を入力してもらう
        ' ========================================================
        If owner Is Nothing Then
            myResult = ShowDialog()
        Else
            myResult = ShowDialog(owner)
        End If


        ' ========================================================
        '   OK以外で終了したのなら何もしない
        ' ========================================================
        If myResult <> Windows.Forms.DialogResult.OK Then
            Return Nothing
        End If


        Return d_myResult
    End Function


    Public Function ShowProperty(ByVal avValue As KEYWORDTYPE, Optional ByRef owner As System.Windows.Forms.IWin32Window = Nothing) As KEYWORDTYPE

        Dim myResult As System.Windows.Forms.DialogResult

        ' ========================================================
        '   デフォルト値を設定
        ' ========================================================
        d_DefaultValue.IsEnable = True
        d_DefaultValue.Value = avValue

        ' ========================================================
        '   ダイアログを表示してキーワード情報を入力してもらう
        ' ========================================================
        If owner Is Nothing Then
            myResult = ShowDialog()
        Else
            myResult = ShowDialog(owner)
        End If


        ' ========================================================
        '   OK以外で終了したのなら何もしない
        ' ========================================================
        If myResult <> Windows.Forms.DialogResult.OK Then
            Return Nothing
        End If

        Return d_myResult
    End Function



    Private Sub OK_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles OK_Button.Click
        ' ========================================================
        '   キーワードの入力チェック
        ' ========================================================
        If System.String.IsNullOrEmpty(txtKeyWord.Text) Then
            MessageBox.Show("キーワードを入力してください", "警告", MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return
        End If


        ' ========================================================
        '   キーワードを戻り値として設定
        ' ========================================================
        d_myResult.Keyword = txtKeyWord.Text


        ' ========================================================
        '   キーワードの検索タイプを戻り値として設定
        ' ========================================================
        If RbSearchRegular.Checked Then d_myResult.Search = SEARCHTYPE.Regular
        If RbSearchWordIns.Checked Then d_myResult.Search = SEARCHTYPE.WordInsensitive
        If RbSearchWord.Checked Then d_myResult.Search = SEARCHTYPE.Word


        ' ========================================================
        '   キーワードの表示種別を戻り値として設定
        ' ========================================================
        d_myResult.KeyShowType = ListKeyType.SelectedItem.ToString


        ' ========================================================
        '   選択OK！
        ' ========================================================
        Me.DialogResult = System.Windows.Forms.DialogResult.OK
        Me.Close()
    End Sub


    Private Sub Cancel_Button_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Cancel_Button.Click
        Me.DialogResult = System.Windows.Forms.DialogResult.Cancel
        Me.Close()
    End Sub


    Private Sub dlgKeyProperty_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        ' ========================================================
        '   念のため値を初期化
        ' ========================================================
        'd_myResult = Nothing   こいつは閉じた後も使用するため初期化しない
        d_DefaultValue.IsEnable = False
        ListKeyType.Items.Clear()
        txtKeyWord.Text = vbNullString
        RbSearchRegular.Checked = True
    End Sub


    Private Sub dlgKeyProperty_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' ========================================================
        '   リストにキーワードを追加して、１個目を選択
        ' ========================================================
        ListKeyType.Items.Add("KeyWord_Type1")
        ListKeyType.Items.Add("KeyWord_Type2")
        ListKeyType.Items.Add("KeyWord_Type3")
        ListKeyType.Items.Add("KeyWord_Type4")
        ListKeyType.Items.Add("KeyWord_Type5")
        ListKeyType.Items.Add("KeyWord_Type6")
        ListKeyType.Items.Add("KeyWord_Type7")
        ListKeyType.Items.Add("KeyWord_Type8")
        ListKeyType.Items.Add("KeyWord_Type9")
        ListKeyType.Items.Add("KeyWord_TypeA")
        ListKeyType.Items.Add("KeyWord_TypeB")
        ListKeyType.Items.Add("KeyWord_TypeC")
        ListKeyType.Items.Add("KeyWord_TypeD")
        ListKeyType.Items.Add("KeyWord_TypeE")
        ListKeyType.Items.Add("KeyWord_TypeF")
        ListKeyType.SelectedIndex = 0

        Application.DoEvents()


        ' ========================================================
        '   デフォルト値が有効であれば設定する
        ' ========================================================
        Try
            If d_DefaultValue.IsEnable Then
                ' ========================================================
                '   キーワードとその表示タイプを設定
                ' ========================================================
                txtKeyWord.Text = d_DefaultValue.Value.Keyword
                ListKeyType.SelectedIndex = ListKeyType.Items.IndexOf(d_DefaultValue.Value.KeyShowType)

                ' ========================================================
                '   キーワードの検索タイプをラジオボタンに設定
                ' ========================================================
                If d_DefaultValue.Value.Search = SEARCHTYPE.Regular Then RbSearchRegular.Checked = True
                If d_DefaultValue.Value.Search = SEARCHTYPE.WordInsensitive Then RbSearchWordIns.Checked = True
                If d_DefaultValue.Value.Search = SEARCHTYPE.Word Then RbSearchWord.Checked = True
            End If

        Catch ex As Exception

        End Try
    End Sub

End Class
