﻿Option Explicit On

Module myModule


    ' ========================================================================
    '   ◆設定データ構造体
    ' ========================================================================
    ''' <summary>設定データ</summary>
    Public Structure MYSETTING
        ''' <summary>TAB幅</summary>
        Dim TabWidth As Integer
        ''' <summary>CSSテキスト</summary>
        Dim CssText As String
        ''' <summary>行番号をつけるかどうか</summary>
        Dim IsWriteLineNo As Boolean
        ''' <summary>ソースコード</summary>
        Dim SourceCode As String
        ''' <summary>キーワード定義</summary>
        Dim XmlKeyWord As System.Xml.XmlNode
    End Structure


    Private Structure MYCONVLIST
        Dim Text As String
        Dim ViewType As String
    End Structure

    Private Structure MYMATCHINFO
        Dim Text As String
        Dim ViewType As String
        Dim StartIndex As Integer
        Dim EndIndex As Integer
    End Structure



    ''' ====================================================================
    ''' = ◆ 関数機能
    ''' ---------------------------------------------------------------------
    ''' <summary>
    '''      <para>設定ファイルを開いて設定データを取得します</para>
    ''' </summary>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 引数
    ''' ---------------------------------------------------------------------
    ''' <param name="avSourceCode">
    '''      <para>変換するソースコードを指定します</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' <param name="avTabSize">
    '''      <para>タブサイズを指定します</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' <param name="avIsWriteLineNo">
    '''      <para>行番号を書くかどうかを指定します</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' <param name="avXmlKeyWord">
    '''      <para>変換キーワード情報が記載されたXMLノードを指定します。</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 戻り値
    ''' ---------------------------------------------------------------------
    ''' <returns>
    '''      <para>HTML変換されたソースコードが返ります</para>
    ''' </returns>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 備考
    ''' ---------------------------------------------------------------------
    ''' <remarks></remarks>
    ''' ========================================================================
    Public Function ConvertSourceToHtml(ByVal avSourceCode As String, ByVal avTabSize As Integer, ByVal avIsWriteLineNo As Boolean, Optional ByVal avXmlKeyWord As System.Xml.XmlNode = Nothing) As String

        Dim myKeyWord As String = vbNullString

        ' ========================================================
        '   見えないリッチテキストボックスを生成し、ソースを貼り付け
        ' ========================================================
        Dim myDummyRichText As System.Windows.Forms.RichTextBox = New System.Windows.Forms.RichTextBox
        myDummyRichText.Text = avSourceCode


        ' ========================================================
        '   見えないリッチテキストボックスの文字色を真っ黒にする
        '   この色がCSSタグの種類を示す(BLUE 0 = タグナシ)
        ' ========================================================
        myDummyRichText.SelectAll()
        myDummyRichText.SelectionColor = System.Drawing.Color.FromArgb(0, 0, 0, 0)


        ' ========================================================
        '   CSSタグの配列を生成する
        '   配列のIndexがリッチテキストボックスのBLUE文字色を示す
        ' ========================================================
        Dim myCssTagList() As String = Nothing





        Try
            ' ========================================================
            '   キーワードリスト[ITEM]を抽出
            ' ========================================================
            Dim myXmlKeyWords As System.Xml.XmlNodeList = avXmlKeyWord.SelectNodes("./ITEM")


            For i As Integer = 0 To myXmlKeyWords.Count - 1

                ' ========================================================
                '   XMLからデータを抽出
                ' ========================================================
                ' ====== キーワードを抽出
                myKeyWord = myXmlKeyWords(i).SelectSingleNode("./KEY").InnerText
                ' ====== 検索方法を抽出
                Dim mySearchType As dlgKeyProperty.SEARCHTYPE = System.Convert.ToInt32(myXmlKeyWords(i).SelectSingleNode("./SEARCHTYPE").InnerText)
                ' ====== CSSタグ名を抽出
                Dim myCssTagName As String = myXmlKeyWords(i).SelectSingleNode("./CSSTAGNAME").InnerText
                ' ====== CSSタグ名が格納される配列のIndex
                Dim myCssTagIndex As Integer = 0

                ' ========================================================
                '   CSSタグリストに同名のタグがなければ、あらたに追加する
                ' ========================================================
                If myCssTagList Is Nothing Then
                    ' ========================================================
                    '   配列が未使用の場合は領域を１コ確保して追加する
                    ' ========================================================
                    ReDim myCssTagList(0)
                    myCssTagList(myCssTagList.Length - 1) = myCssTagName
                    myCssTagIndex = 0
                Else
                    ' ========================================================
                    '   配列をタグ名で検索し、そのIndexを取得する
                    ' ========================================================
                    myCssTagIndex = System.Array.IndexOf(myCssTagList, myCssTagName)
                    ' ====== 見つからなかったら-1が返るらしい
                    If myCssTagIndex < 0 Then
                        ' ========================================================
                        '   見つからなかった場合、領域を１コ確保して追加する
                        ' ========================================================
                        ReDim Preserve myCssTagList(myCssTagList.Length)
                        myCssTagList(myCssTagList.Length - 1) = myCssTagName
                        myCssTagIndex = myCssTagList.Length - 1
                    End If

                End If

                ' ========================================================
                '   ソースコードに含まれるキーワードを検索し、その文字をマークアップ(色付け)する
                ' ========================================================
                If Not System.String.IsNullOrEmpty(myCssTagName) Then
                    Select Case mySearchType
                        Case dlgKeyProperty.SEARCHTYPE.Regular
                            ' ========================================================
                            '   正規表現でのキーワード検索
                            ' ========================================================
                            ' ====== キーワードを元に正規表現パターンを生成
                            Dim myRegex As System.Text.RegularExpressions.Regex = New System.Text.RegularExpressions.Regex(myKeyWord)
                            ' ====== 正規表現検索を行い、マッチリストを生成
                            Dim myMatchList As System.Text.RegularExpressions.MatchCollection = myRegex.Matches(myDummyRichText.Text)

                            ' ========================================================
                            '   正規表現にマッチした文字列リストを全部マークアップする
                            ' ========================================================
                            If Not myMatchList Is Nothing Then
                                For j As Integer = 0 To myMatchList.Count - 1

                                    ' ========================================================
                                    '   マッチ部分の文字列をマークアップします。タグ名配列の Index + 1 が文字色になります。
                                    '   Index + 1 としているのは、Index = 0 のとき、色がついていないモノと区別がつかないためです。
                                    ' ========================================================
                                    With myMatchList(j)
                                        MarkUpRichText(myDummyRichText, .Index, .Length, myCssTagIndex + 1)
                                    End With
                                Next
                            End If


                        Case dlgKeyProperty.SEARCHTYPE.Word
                            ' ========================================================
                            '   キーワードの単語検索(大文字/小文字区別)
                            ' ========================================================
                            ' ====== 単語にマッチする正規表現パターンを生成
                            Dim myRegex As System.Text.RegularExpressions.Regex = New System.Text.RegularExpressions.Regex("(\b)" + "(" + myKeyWord + ")" + "(\b)")
                            ' ====== 正規表現検索を行い、マッチリストを生成
                            Dim myMatchList As System.Text.RegularExpressions.MatchCollection = myRegex.Matches(myDummyRichText.Text)
                            ' ※補足 : 正規表現の "\b" は、単語の境界を示します
                            '        : $2 は、グループ化された(括弧で囲んだ)パターンの2番目にマッチした文字を示します

                            ' ========================================================
                            '   正規表現にマッチした文字列リストを全部マークアップします
                            ' ========================================================
                            If Not myMatchList Is Nothing Then
                                For j As Integer = 0 To myMatchList.Count - 1

                                    ' ========================================================
                                    '   マッチ部分の文字列をマークアップします。タグ名配列の Index + 1 が文字色になります。
                                    '   Index + 1 としているのは、Index = 0 のとき、色がついていないモノと区別がつかないためです。
                                    ' ========================================================
                                    With myMatchList(j).Groups(2)
                                        MarkUpRichText(myDummyRichText, .Index, .Length, myCssTagIndex + 1)
                                    End With

                                Next
                            End If




                        Case Else
                            ' ========================================================
                            '   キーワードの単語検索(大文字/小文字同一視)
                            ' ========================================================
                            ' ====== 単語にマッチする正規表現パターンを生成
                            Dim myRegex As System.Text.RegularExpressions.Regex = New System.Text.RegularExpressions.Regex("(\b)" + "(" + myKeyWord + ")" + "(\b)", System.Text.RegularExpressions.RegexOptions.IgnoreCase)
                            Dim myMatchList As System.Text.RegularExpressions.MatchCollection = myRegex.Matches(myDummyRichText.Text)
                            ' ※補足 : 正規表現の "\b" は、単語の境界を示します
                            '        : $2 は、グループ化された(括弧で囲んだ)パターンの2番目にマッチした文字を示します

                            ' ========================================================
                            '   正規表現にマッチした文字列リストを全部マークアップする
                            ' ========================================================
                            If Not myMatchList Is Nothing Then
                                For j As Integer = 0 To myMatchList.Count - 1

                                    ' ========================================================
                                    '   マッチ部分の文字列をマークアップします。タグ名配列の Index + 1 が文字色になります。
                                    '   Index + 1 としているのは、Index = 0 のとき、色がついていないモノと区別がつかないためです。
                                    ' ========================================================
                                    With myMatchList(j).Groups(2)
                                        MarkUpRichText(myDummyRichText, .Index, .Length, myCssTagIndex + 1)
                                    End With
                                Next
                            End If
                    End Select
                End If
            Next




        Catch ex As Exception
            If MessageBox.Show(myKeyWord + " は不正な正規表現なため正常に変換できませんでした." + vbCrLf + "続けますか？", "警告", MessageBoxButtons.YesNo, MessageBoxIcon.Error) = DialogResult.No Then
                Return vbNullString
            End If
        End Try

        ' ========================================================
        '   HTMLタグをHTMLタグの文字コードに変換します
        ' ========================================================
        ReplaceHtmlTagCode(myDummyRichText)


        ' ========================================================
        '   全ての文字を一文字ずつ走査し、色がついている部分にタグをつけます。
        ' ========================================================
        With myDummyRichText
            ' ====== 先頭を1文字選択
            .Select(0, 1)

            While .SelectionStart < .TextLength

                ' ========================================================
                '   前準備
                ' ========================================================
                ' ====== カーソル位置の色(BLUE)
                Dim mySelectionBlue As Integer

                ' ====== カーソル位置(選択終了) 
                Dim mySelectionEnd As Integer

                ' ====== カーソル位置(選択開始)
                Dim mySelectionStart As Integer = .SelectionStart

                ' ====== カーソル位置の色(BLUE)
                mySelectionBlue = .SelectionColor.B

                ' ====== 念のための異常排他
                If (mySelectionBlue - 1) >= myCssTagList.Length Then
                    Throw New ApplicationException("置換用文字コードの不具合のため正常動作できませんでした")
                End If


                ' ========================================================
                '   選択位置が色付けされている(BLUEがゼロでない)のであれば、
                '   どこまで色付けされているか走査します。
                ' ========================================================
                If 0 <> mySelectionBlue Then
                    ' ====== 選択開始、終了、開始位置の色を記憶
                    mySelectionStart = .SelectionStart
                    mySelectionEnd = .SelectionStart
                    mySelectionBlue = .SelectionColor.B

                    ' ========================================================
                    '   色付けされている文字列の終了位置を算出
                    ' ========================================================
                    For i As Integer = mySelectionStart To .TextLength
                        ' ====== 次の一文字選択
                        .Select(i, 1)
                        mySelectionEnd = i

                        ' ====== 選択した一文字の色が、前の色と違うか調べる
                        If mySelectionBlue <> .SelectionColor.B Then
                            ' ====== 色が違うので、範囲はここまでらしい
                            mySelectionEnd = i
                            System.Diagnostics.Debug.Print("selected colored text:" + myDummyRichText.Text.Substring(mySelectionStart, mySelectionEnd - mySelectionStart))
                            Exit For
                        End If
                    Next

                    ' ========================================================
                    '   色付けされているテキストを選択して色づけを解除
                    ' ========================================================
                    If (mySelectionEnd - mySelectionStart) <= 0 Then Throw New ApplicationException("置換用文字コードの終了位置走査に不具合があったため正常動作できませんでした")
                    .Select(mySelectionStart, (mySelectionEnd - mySelectionStart))
                    .SelectionColor = System.Drawing.Color.FromArgb(0, 0, 0, 0)

                    ' ========================================================
                    '   選択されていたテキストを改行区切りに分解
                    ' ========================================================
                    Dim mySelectionString As String = vbNullString
                    Dim myStringArray() As String = .SelectedText.Split(vbLf)

                    ' ========================================================
                    '   選択テキストを１行ごとにHTMLタギング:<span class="xxx">で囲む
                    ' ========================================================
                    For i As Integer = 0 To (myStringArray.Length - 1)
                        mySelectionString += "<span class=""" + myCssTagList(mySelectionBlue - 1) + """>" + myStringArray(i) + "</span>"
                        If i < (myStringArray.Length - 1) Then
                            mySelectionString += vbLf
                        End If
                    Next

                    System.Diagnostics.Debug.Print("Tagging Selected Text:" + mySelectionString)

                    ' ====== 選択テキストを書き換える前に、次の開始位置を再計算
                    mySelectionStart += mySelectionString.Length

                    ' ====== 選択テキストをタグ囲い文字で上書き
                    .SelectedText = mySelectionString

                    ' ここで.SelectionStartが初期化される
                End If

                ' ========================================================
                '   次の一文字を選択
                ' ========================================================
                .Select(mySelectionStart + 1, 1)
            End While
        End With

        Dim myString As String = vbNullString

        ' ========================================================
        '   行番号を書き込みます
        ' ========================================================
        If avIsWriteLineNo Then
            myString = WriteLineNoTag(myDummyRichText)
        Else
            myString = myDummyRichText.Text
        End If

        ' ========================================================
        '   タブをスペースに変換
        ' ========================================================
        Dim myWhiteSpace As String = Space(avTabSize)
        myString = myString.Replace(Microsoft.VisualBasic.ControlChars.Tab, myWhiteSpace)


        ' ========================================================
        '   リッチテキストのラインフィード(vbLf)を改行(vbCrLf)に変換
        ' ========================================================
        '   myString = myString.Replace(Microsoft.VisualBasic.vbLf, Microsoft.VisualBasic.vbCrLf)


        ' ========================================================
        '   <pre>タグをつけて返す
        ' ========================================================
        Return ("<pre>" + vbCrLf + myString + vbCrLf + "</pre>")
    End Function



    ''' ====================================================================
    ''' = ◆ 関数機能
    ''' ---------------------------------------------------------------------
    ''' <summary>
    '''      <para>リッチテキストの指定テキスト分部に色をつけます</para>
    ''' </summary>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 引数
    ''' ---------------------------------------------------------------------
    ''' <param name="apRichTextBox">
    '''      <para>対象のリッチテキストボックス</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' <param name="avStartIndex">
    '''      <para>マークアップ開始位置</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' <param name="avSelectLength">
    '''      <para>マークアップ文字列長</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' <param name="avColorCodeBlue">
    '''      <para>文字色(BLUEのみ)</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 戻り値
    ''' ---------------------------------------------------------------------
    ''' <returns>
    '''      <para>マークアップ実行した場合trueを返します。</para>
    '''      <para>指定テキスト内に既に色がついている文字がある場合はマークアップ実行しません。</para>
    ''' </returns>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 備考
    ''' ---------------------------------------------------------------------
    ''' <remarks>
    '''      <para>既に色がついている場合はマークアップ実行しません。</para>
    ''' </remarks>
    ''' ========================================================================
    Public Function MarkUpRichText(ByRef apRichTextBox As System.Windows.Forms.RichTextBox, ByVal avStartIndex As Integer, ByVal avSelectLength As Integer, ByVal avColorCodeBlue As Byte) As Boolean

        ' ====== マークアップ実行すると前提
        Dim myDoMarkUp As Boolean = True

        With apRichTextBox
            ' ========================================================
            '   マッチ部分の文字色BLUEがゼロでなければ、それは既に他の
            '   優先度が高いキーワードでマークアップ済みなので何もしない。
            ' ========================================================
            For i As Integer = avStartIndex To (avStartIndex + avSelectLength) - 1
                ' ========================================================
                '   1文字ずつ選択し、BLUEに色がついているか確認
                ' ========================================================
                apRichTextBox.Select(i, 1)
                If 0 <> apRichTextBox.SelectionColor.B Then
                    ' ====== 文字色がついていたらループ脱出。
                    myDoMarkUp = False
                    Exit For
                End If
            Next

            ' ========================================================
            '   マッチ部分の文字列をマークアップする
            ' ========================================================
            If myDoMarkUp Then
                System.Diagnostics.Debug.Print("Markup Word:" + .ToString)
                ' ====== テキストボックスのマッチしたワードを選択
                .Select(avStartIndex, avSelectLength)
                ' ====== テキストボックスのマッチしたワードに色をつける
                .SelectionColor = System.Drawing.Color.FromArgb(0, 0, 0, avColorCodeBlue)
            End If
        End With


        Return myDoMarkUp
    End Function


    ''' ====================================================================
    ''' = ◆ 関数機能
    ''' ---------------------------------------------------------------------
    ''' <summary>
    '''      <para>HTMLタグをHTML文字コードに変換します</para>
    ''' </summary>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 引数
    ''' ---------------------------------------------------------------------
    ''' <param name="apRichTextBox">
    '''      <para>変換するソースコードが記載されたリッチテキストボックスを指定します</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 備考
    ''' ---------------------------------------------------------------------
    ''' <remarks></remarks>
    ''' ========================================================================
    Public Sub ReplaceHtmlTagCode(ByRef apRichTextBox As System.Windows.Forms.RichTextBox)
        apRichTextBox.Rtf = apRichTextBox.Rtf.Replace("<", "&#60;")
        apRichTextBox.Rtf = apRichTextBox.Rtf.Replace(">", "&#62;")
    End Sub



    ''' ====================================================================
    ''' = ◆ 関数機能
    ''' ---------------------------------------------------------------------
    ''' <summary>
    '''      <para>行番号タグを挿入した文字列を返します</para>
    ''' </summary>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 引数
    ''' ---------------------------------------------------------------------
    ''' <param name="apRichTextBox">
    '''      <para>対象のリッチテキストボックス</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 戻り値
    ''' ---------------------------------------------------------------------
    ''' <returns>
    '''      <para>行番号タグを挿入した文字列</para>
    ''' </returns>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 備考
    ''' ---------------------------------------------------------------------
    ''' <remarks>
    '''      <para></para>
    ''' </remarks>
    ''' ========================================================================
    Public Function WriteLineNoTag(ByRef apRichTextBox As System.Windows.Forms.RichTextBox) As String
        Dim myString As String = vbNullString

        With apRichTextBox
            ' ========================================================
            '   各行を<li></li>タグで囲む
            '   ※番号付きリスト <OL> <LI>
            ' ========================================================
            For i As Integer = 0 To .Lines.Length - 1
                myString = myString + "<li class=""" + IIf((i Mod 2) = 0, "LINE_ODD", "LINE_EVEN") + """>" + .Lines(i) + "</li>" + vbCrLf
            Next
        End With


        ' ========================================================
        '   全体を<OL></OL>タグ:番号付きリスト で囲む
        ' ========================================================
        myString = "<ol class=""CODE_LINE"">" + myString + "</ol>"

        Return myString
    End Function


    ''' ====================================================================
    ''' = ◆ 関数機能
    ''' ---------------------------------------------------------------------
    ''' <summary>
    '''      <para>設定ファイルを開いて設定データを取得します</para>
    ''' </summary>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 引数
    ''' ---------------------------------------------------------------------
    ''' <param name="avPath">
    '''      <para>設定ファイル(XML)のパスを指定します</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' <param name="avIsFileNameOnly">
    '''      <para>省略可能です。avPathがフルパスを示す場合falseにしてください。</para>
    '''      <para>trueにした場合、実行ディレクトリを対象にします。</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 戻り値
    ''' ---------------------------------------------------------------------
    ''' <returns>
    '''      <para>読み込んだ設定データが返ります</para>
    ''' </returns>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 備考
    ''' ---------------------------------------------------------------------
    ''' <remarks></remarks>
    ''' ========================================================================
    Public Function LoadSettingInfo(ByVal avPath As String, Optional ByVal avIsFileNameOnly As Boolean = True) As MYSETTING

        Dim myXmlDoc As System.Xml.XmlDocument = New System.Xml.XmlDocument
        Dim mySetting As MYSETTING = Nothing
        Dim myPath As String = vbNullString
        Dim myBytes() As Byte = Nothing


        ' ========================================================
        '   実行パス + 設定ファイル(XML)を合成してフルパスを生成
        ' ========================================================
        If avIsFileNameOnly Then
            myPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location)
            myPath = System.IO.Path.Combine(myPath, avPath)
        End If


        ' ========================================================
        '   XMLを開いて設定データをロードします
        ' ========================================================
        Try
            ' ====== XMLをロード
            myXmlDoc.Load(myPath)

            ' ====== ルートノードを選択
            Dim myXmlRootNode = myXmlDoc.SelectSingleNode("./ROOT")

            ' ========================================================
            '   BASE64エンコードされている[CSS]をXMLから取得してデコードする
            ' ========================================================
            myBytes = System.Convert.FromBase64String(myXmlRootNode.SelectSingleNode("./CSS").InnerText)
            mySetting.CssText = System.Text.Encoding.UTF8.GetString(myBytes)


            ' ========================================================
            '   BASE64エンコードされている[ソースコード]をXMLから取得してデコードする
            ' ========================================================
            myBytes = System.Convert.FromBase64String(myXmlRootNode.SelectSingleNode("./SOURCE").InnerText)
            mySetting.SourceCode = System.Text.Encoding.UTF8.GetString(myBytes)

            ' ========================================================
            '   XMLから[TAB]幅を取得する
            ' ========================================================
            Try
                Dim myString = myXmlRootNode.SelectSingleNode("./TAB").InnerText
                If Not Integer.TryParse(myString, mySetting.TabWidth) Then Throw New ApplicationException
            Catch ex As Exception
                mySetting.TabWidth = 4
            End Try

            ' ========================================================
            '   XMLから[LINENO]を取得する
            ' ========================================================
            Try
                Dim myString = myXmlRootNode.SelectSingleNode("./LINENO").InnerText
                If Not Boolean.TryParse(myString, mySetting.IsWriteLineNo) Then Throw New ApplicationException
            Catch ex As Exception
                mySetting.IsWriteLineNo = True
            End Try


            ' ========================================================
            '   キーワードノードを抽出
            ' ========================================================
            Try
                mySetting.XmlKeyWord = myXmlDoc.ImportNode(myXmlRootNode.SelectSingleNode("./KEYWORDS"), True)
            Catch ex As Exception
                mySetting.XmlKeyWord = Nothing
            End Try
        Catch ex As Exception
            Return Nothing
        End Try

        Return mySetting
    End Function





    ''' ====================================================================
    ''' = ◆ 関数機能
    ''' ---------------------------------------------------------------------
    ''' <summary>
    '''      <para>設定ファイルに設定データを保存します</para>
    ''' </summary>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 引数
    ''' ---------------------------------------------------------------------
    ''' <param name="avPath">
    '''      <para>設定ファイル(XML)のパスを指定します</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' <param name="avIsFileNameOnly">
    '''      <para>省略可能です。avPathがフルパスを示す場合falseにしてください。</para>
    '''      <para>trueにした場合、実行ディレクトリを対象にします。</para>
    ''' </param>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 戻り値
    ''' ---------------------------------------------------------------------
    ''' <returns>
    '''      <para>成功した場合trueが返ります</para>
    ''' </returns>
    ''' ---------------------------------------------------------------------
    ''' = ◆ 備考
    ''' ---------------------------------------------------------------------
    ''' <remarks></remarks>
    ''' ========================================================================
    Public Function SaveSettingInfo(ByVal avSettingInfo As MYSETTING, ByVal avPath As String, Optional ByVal avIsFileNameOnly As Boolean = True) As Boolean

        Dim myXmlDoc As System.Xml.XmlDocument = New System.Xml.XmlDocument
        Dim myPath As String = vbNullString


        ' ========================================================
        '   実行パス + 設定ファイル(XML)を合成してフルパスを生成
        ' ========================================================
        If avIsFileNameOnly Then
            myPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location)
            myPath = System.IO.Path.Combine(myPath, avPath)
        End If


        ' ========================================================
        '   XMLを開いて設定データをロードします
        ' ========================================================
        Dim myXmlRootNode As System.Xml.XmlNode = Nothing
        Try
            Try
                ' ========================================================
                '   既存のファイルをロードし、ルートノードを選択
                ' ========================================================
                myXmlDoc.Load(myPath)
                myXmlRootNode = myXmlDoc.SelectSingleNode("./ROOT")
                If myXmlRootNode Is Nothing Then Throw New ApplicationException
            Catch ex As Exception

                ' ========================================================
                '   ルートが見つからない or ファイルが開けないなら、新しく生成
                ' ========================================================
                myXmlDoc.LoadXml("<?xml version=""1.0"" encoding=""utf-8"" ?>" & "<ROOT/>")
                myXmlRootNode = myXmlDoc.SelectSingleNode("./ROOT")
            End Try

            Dim myXmlNode As System.Xml.XmlNode = Nothing
            Dim myBytes() As Byte = Nothing



            ' ========================================================
            '   CSSノードが存在するなら選択し、ないなら生成する
            ' ========================================================
            myXmlNode = myXmlRootNode.SelectSingleNode("./CSS")
            If myXmlNode Is Nothing Then
                myXmlNode = myXmlRootNode.AppendChild(myXmlDoc.CreateElement("CSS"))
            End If

            ' ========================================================
            '   [CSS]をBASE64にエンコードしてXMLノードに追加(不正な文字コードなどでXMLが壊れないように)
            ' ========================================================
            If avSettingInfo.CssText <> vbNullString Then
                myBytes = System.Text.Encoding.UTF8.GetBytes(avSettingInfo.CssText)
                myXmlNode.InnerText = System.Convert.ToBase64String(myBytes)
            Else
                myXmlNode.InnerText = vbNullString
            End If



            ' ========================================================
            '   [SOURCE]ノードが存在するなら選択し、ないなら生成する
            ' ========================================================
            myXmlNode = myXmlRootNode.SelectSingleNode("./SOURCE")
            If myXmlNode Is Nothing Then
                myXmlNode = myXmlRootNode.AppendChild(myXmlDoc.CreateElement("SOURCE"))
            End If

            ' ========================================================
            '   [SOURCE]をBASE64にエンコードしてXMLノードに追加(不正な文字コードなどでXMLが壊れないように)
            ' ========================================================
            If avSettingInfo.SourceCode <> vbNullString Then
                myBytes = System.Text.Encoding.UTF8.GetBytes(avSettingInfo.SourceCode)
                myXmlNode.InnerText = System.Convert.ToBase64String(myBytes)
            Else
                myXmlNode.InnerText = vbNullString
            End If



            ' ========================================================
            '   [TAB]ノードが存在するなら選択し、ないなら生成して保存する
            ' ========================================================
            myXmlNode = myXmlRootNode.SelectSingleNode("./TAB")
            If myXmlNode Is Nothing Then
                myXmlNode = myXmlRootNode.AppendChild(myXmlDoc.CreateElement("TAB"))
            End If
            myXmlNode.InnerText = avSettingInfo.TabWidth.ToString


            ' ========================================================
            '   [LINENO]ノードが存在するなら選択し、ないなら生成して保存する
            ' ========================================================
            myXmlNode = myXmlRootNode.SelectSingleNode("./LINENO")
            If myXmlNode Is Nothing Then
                myXmlNode = myXmlRootNode.AppendChild(myXmlDoc.CreateElement("LINENO"))
            End If
            myXmlNode.InnerText = avSettingInfo.IsWriteLineNo.ToString


            ' ========================================================
            '   [KEYWORDS]ノードが存在するなら削除し、新しい[KEYWORDS]ノードを追加
            ' ========================================================
            myXmlNode = myXmlRootNode.SelectSingleNode("./KEYWORDS")
            If Not myXmlNode Is Nothing Then
                myXmlRootNode.RemoveChild(myXmlNode)
            End If
            myXmlRootNode.AppendChild(myXmlDoc.ImportNode(avSettingInfo.XmlKeyWord, True))


            ' ========================================================
            '   XMLを保存
            ' ========================================================
            myXmlDoc.Save(myPath)

        Catch ex As Exception
            Return False
        End Try

        Return True
    End Function

End Module
